@@ -7,11 +7,8 @@
 %% cloudi_service callbacks
 -export([cloudi_service_init/4,
          cloudi_service_handle_request/11,
-         cloudi_service_handle_info/3,
          cloudi_service_terminate/3]).
 
--include_lib("cloudi_core/include/cloudi_logger.hrl").
-
 -record(state,
     {
     }).
@@ -25,9 +22,5 @@ cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, _Request,
                               #state{} = State, _Dispatcher) ->
     {reply, <<"Hello World!">>, State}.
 
-cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
-
-cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
+cloudi_service_terminate(_Reason, _Timeout, _State) ->
     ok.
