@@ -30,7 +30,7 @@
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2014-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @version 1.7.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_http_client).
@@ -629,7 +629,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
              ["head", "get", "put", "post", "trace", "options", "delete"],
              Profile1}
     end,
-    false = lists:member($*, Prefix),
+    false = cloudi_service_name:pattern(Prefix),
     [cloudi_service:subscribe(Dispatcher, [$/ | Method]) || Method <- Methods],
     true = ((DebugLevel =:= trace) orelse
             (DebugLevel =:= debug) orelse
@@ -685,7 +685,7 @@ headers_external_incoming(<<>>) ->
 headers_external_incoming([] = ResponseInfo) ->
     ResponseInfo;
 headers_external_incoming([{K, V} | _] = ResponseInfo) ->
-    % K/V possibly from cloudi_service_http_cowboy HTTP request headers
+    % K/V possibly from cloudi_service_http_cowboy1 HTTP request headers
     true = is_binary(K),
     true = is_binary(V),
     ResponseInfo;
@@ -705,7 +705,7 @@ headers_request(RequestInfo, internal) ->
         [] ->
             RequestInfo;
         [{K, V} | _] ->
-            % K/V possibly from cloudi_service_http_cowboy HTTP request headers
+            % K/V possibly from cloudi_service_http_cowboy1 HTTP request headers
             true = is_binary(K),
             true = is_binary(V),
             RequestInfo
@@ -727,7 +727,7 @@ headers_request_filter(Headers0) ->
         {<<"host">>,                     undefined},
         {<<"url-path">>,                 undefined},
         % removed parameters
-        % (possible if coming from cloudi_service_http_cowboy)
+        % (possible if coming from cloudi_service_http_cowboy1)
         {<<"peer">>,                     undefined},
         {<<"peer-port">>,                undefined},
         {<<"source-address">>,           undefined},
