@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,13 +30,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2014-2018 Michael Truog
+%%% @version 1.7.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_oauth1).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -108,6 +108,29 @@
         prefix_length :: pos_integer()
     }).
 
+% for features specific to Erlang/OTP version 21.x (and later versions)
+-ifdef(ERLANG_OTP_VERSION_19).
+-else.
+-ifdef(ERLANG_OTP_VERSION_20).
+-else.
+-ifdef(OTP_RELEASE).
+-define(ERLANG_OTP_VERSION_21_FEATURES, true).
+-else.
+-error("Erlang/OTP version invalid").
+-endif.
+-endif.
+-endif.
+
+% Get the stacktrace in a way that is backwards compatible
+-ifdef(ERLANG_OTP_VERSION_21_FEATURES).
+-define(STACKTRACE(ErrorType, Error, ErrorStackTrace),
+        ErrorType:Error:ErrorStackTrace ->).
+-else.
+-define(STACKTRACE(ErrorType, Error, ErrorStackTrace),
+        ErrorType:Error ->
+            ErrorStackTrace = erlang:get_stacktrace(),).
+-endif.
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -198,7 +221,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
             true = is_boolean(DebugDB0),
             DebugDB0
     end,
-    false = lists:member($*, Prefix),
+    false = cloudi_service_name:pattern(Prefix),
     % endpoints based on http://tools.ietf.org/html/rfc5849#section-1.2
     ok = cloudi_service:subscribe(Dispatcher, "initiate/post"),
     ok = cloudi_service:subscribe(Dispatcher, "authorize/get"),
@@ -252,10 +275,9 @@ cloudi_service_handle_info({tokens_clean, TokensClean} = Request,
                       Request),
     {noreply, State};
 cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
-cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
+cloudi_service_terminate(_Reason, _Timeout, _State) ->
     ok.
 
 %%%------------------------------------------------------------------------
@@ -342,7 +364,7 @@ request("verify*", Name, Pattern, RequestHeaders, Request, Timeout,
         #state{url_host = URLHost,
                host = Host} = State, Dispatcher) ->
     [NextName] = cloudi_service_name:parse(Name, Pattern),
-    Method = string:to_upper(cloudi_string:afterr($/, NextName)),
+    Method = cloudi_string:uppercase(cloudi_string:afterr($/, NextName)),
     case url(Method, RequestHeaders, URLHost, Host, Request) of
         {ok, URL} ->
             case oauth_parameters(Method, RequestHeaders, Request) of
@@ -363,7 +385,7 @@ url(Method, RequestHeaders, URLHost, Host, Request) ->
     case cloudi_key_value:find(<<"host">>, RequestHeaders) of
         {ok, Host} when Method == "GET" ->
             {ok, URLHost ++ erlang:binary_to_list(URLPath) ++ "?" ++
-                 erlang:binary_to_list(cloudi_x_cow_qs:qs(Request))};
+                 erlang:binary_to_list(cloudi_x_cow1_qs:qs(Request))};
         {ok, Host} ->
             {ok, URLHost ++ erlang:binary_to_list(URLPath)};
         {ok, HostInvalid} ->
@@ -424,10 +446,10 @@ oauth_verify(Signature, Method, URL, Params,
             % invalid oauth_signature
             false
     catch
-        _:Error ->
+        ?STACKTRACE(ErrorType, Error, ErrorStackTrace)
             % implementation error?
-            ?LOG_ERROR("oauth error: ~p~n ~p",
-                       [Error, erlang:get_stacktrace()]),
+            ?LOG_ERROR("oauth failed ~p ~p~n ~p",
+                       [ErrorType, Error, ErrorStackTrace]),
             false
     end.
 
@@ -523,12 +545,7 @@ request_initiate_store(Realm, ConsumerKey, SignatureMethod,
     % provide request token and secret
     TokenRequest = token_request(State),
     TokenRequestSecret = token_request_secret(State),
-    {CallbackURL, CallbackQS} = case string:tokens(Callback, "?") of
-        [CallbackURLValue, CallbackQSValue] ->
-            {CallbackURLValue, CallbackQSValue};
-        [_] ->
-            {Callback, ""}
-    end,
+    {CallbackURL, CallbackQS} = cloudi_string:splitl($?, Callback, input),
     case DatabaseModule:token_request_store(Dispatcher, Database,
                                             Realm, ConsumerKey,
                                             SignatureMethod, ClientSharedSecret,
@@ -695,7 +712,7 @@ callback_merge_qs_filter_request([{K, _} = Entry |
 callback_merge(CallbackURL, CallbackQS, RequestQS, Verifier) ->
     NewCallbackQS = callback_merge_qs_filter_callback(CallbackQS),
     NewRequestQS = callback_merge_qs_filter_request(RequestQS, CallbackQS),
-    QS = cloudi_x_cow_qs:qs(NewCallbackQS ++ NewRequestQS ++
+    QS = cloudi_x_cow1_qs:qs(NewCallbackQS ++ NewRequestQS ++
                    [{<<"oauth_verifier">>, Verifier}]),
     <<CallbackURL/binary, $?, QS/binary>>.
 
@@ -733,7 +750,7 @@ request_authorize(RequestQS, TokenRequest, Timeout,
             end;
         {ok, CallbackURL, CallbackQS} ->
             Callback = callback_merge(CallbackURL,
-                                      cloudi_x_cow_qs:parse_qs(CallbackQS),
+                                      cloudi_x_cow1_qs:parse_qs(CallbackQS),
                                       RequestQS, Verifier),
             case DatabaseModule:token_request_update(Dispatcher, Database,
                                                      TokenRequest, Verifier,
