@@ -42,11 +42,14 @@
 %%% The amount of time the service request is persisted is always limited by
 %%% the timeout of the service request.  Tracking the time taken by a
 %%% service request depends on the Erlang VM time-keeping being dependable
-%%% which depends on the OS time-keeping not varying wildly
-%%% (otherwise service requests may timeout quicker due to the system time
-%%%  moving into the future).  If the fault_isolation service argument is
-%%% set to 'both', the original service request timeout value will be used
-%%% for the service request send to the destination and the service request
+%%% which depends on the OS time-keeping not varying wildly, if
+%%% fault_isolation is set to 'both' and an Erlang VM restart causes an old
+%%% queue file to be used (the new Erlang VM OS process will use the
+%%% new OS time to determine if service requests in the old queue file have
+%%% timed-out based on each service request timeout value).
+%%% If the fault_isolation service argument is set to 'both',
+%%% the original service request timeout value will be used for the
+%%% service request send to the destination and the service request
 %%% send containing the response.
 %%%
 %%% If the retry service argument is set higher than 0, any retry attempts
@@ -60,15 +63,16 @@
 %%% (i.e., without depending on the timeout elapsing locally, despite the
 %%%  timeout being a small value), it is common to set the service
 %%% configuration options request_timeout_immediate_max and
-%%% response_timeout_immediate_max to 0.  If the retry_delay service argument
-%%% is used, setting the request_name_lookup service configuration option to
-%%% async is best if the service request destinations are expected to rarely
-%%% be present (i.e., only appear when anticipating the receive of data).
+%%% response_timeout_immediate_max to 'limit_min'.  If the retry_delay
+%%% service argument is used, setting the request_name_lookup
+%%% service configuration option to async is best if the service request
+%%% destinations are expected to rarely be present
+%%% (i.e., only appear when anticipating the receive of data).
 %%% @end
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -88,13 +92,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2017 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2014-2018 Michael Truog
+%%% @version 1.7.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_queue).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -118,6 +122,17 @@
         % (if the response size is less than or equal to the request size
         %  this is never a problem because the space is reused)
 -define(DEFAULT_COMPRESSION,                    0). % zlib compression 0..9
+-define(DEFAULT_CHECKSUM,               undefined).
+        % Add a checksum to each data chunk in the queue file.
+        % A checksum is not used on the whole queue file.
+        % This is not necessary if your filesystem already
+        % uses checksums (e.g., Btrfs (crc32 variation) and ZFS (sha256)).
+        % Without a checksum it isn't possible to be sure the
+        % data on disk isn't corrupt (once it is recovered after a crash).
+        % All disk writes are done as atomic actions with rename use,
+        % so corruption would be due to an OS failure or hardware failure.
+        % Valid values are:
+        % crc32, md5, ripemd160, sha, sha224, sha256, sha384, sha512
 -define(DEFAULT_RETRY,                          0).
 -define(DEFAULT_RETRY_DELAY,                    0). % milliseconds
 -define(DEFAULT_FAULT_ISOLATION,      destination). % | both
@@ -187,17 +202,23 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
         {file,                         ?DEFAULT_FILE},
         {file_size_limit,   ?DEFAULT_FILE_SIZE_LIMIT},
         {compression,           ?DEFAULT_COMPRESSION},
+        {checksum,                 ?DEFAULT_CHECKSUM},
         {retry,                       ?DEFAULT_RETRY},
         {retry_delay,           ?DEFAULT_RETRY_DELAY},
         {fault_isolation,   ?DEFAULT_FAULT_ISOLATION}],
-    [FilePath, FileSizeLimit, Compression, Retry, RetryDelay,
+    [FilePath, FileSizeLimit, Compression, Checksum, Retry, RetryDelay,
      Mode] = cloudi_proplists:take_values(Defaults, Args),
-    false = cloudi_x_trie:is_pattern(Prefix),
+    false = cloudi_service_name:pattern(Prefix),
     true = is_list(FilePath) andalso is_integer(hd(FilePath)),
     true = is_integer(FileSizeLimit) andalso
            (FileSizeLimit >= 1) andalso (FileSizeLimit =< 18014398509481983),
     true = is_integer(Compression) andalso
            (Compression >= 0) andalso (Compression =< 9),
+    true = (Checksum =:= undefined) orelse
+           (Checksum =:= crc32) orelse (Checksum =:= md5) orelse
+           (Checksum =:= ripemd160) orelse (Checksum =:= sha) orelse
+           (Checksum =:= sha224) orelse (Checksum =:= sha256) orelse
+           (Checksum =:= sha384) orelse (Checksum =:= sha512),
     true = is_integer(Retry) andalso (Retry >= 0),
     true = is_integer(RetryDelay) andalso
            (RetryDelay >= 0) andalso (RetryDelay =< 4294967295),
@@ -216,6 +237,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
     Logging = cloudi_write_ahead_logging:new(QueueFilePath,
                                              FileSizeLimit * 1024,
                                              Compression,
+                                             Checksum,
                                              RetryF),
     cloudi_service:subscribe(Dispatcher, "*"),
     {ok, #state{service = Service,
@@ -405,10 +427,9 @@ cloudi_service_handle_info({retry_delay, QueueTransId},
     % due to its timeout value
     {noreply, State#state{logging = NewLogging}};
 cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
-cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
+cloudi_service_terminate(_Reason, _Timeout, _State) ->
     ok.
 
 %%%------------------------------------------------------------------------
