@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2013-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2013-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,13 +29,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2013-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2013-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(supervisor_cpg_spawn).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(gen_server).
 
@@ -370,15 +370,5 @@ extract_child(#state_nomad{pid = Pid,
 
 -spec timestamp() -> erlang:timestamp().
 
--ifdef(ERLANG_OTP_VERSION_16).
-timestamp() ->
-    erlang:now().
--else.
--ifdef(ERLANG_OTP_VERSION_17).
-timestamp() ->
-    erlang:now().
--else. % necessary for Erlang >= 18.0
 timestamp() ->
     erlang:timestamp().
--endif.
--endif.
