@@ -4,7 +4,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2014-2017 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -25,7 +25,7 @@
 // DEALINGS IN THE SOFTWARE.
 //
 
-require dirname(__FILE__) . '/../../api/php/CloudI.php';
+require 'CloudI.php';
 
 define('DESTINATION', '/tests/msg_size/erlang');
 
